//Global variables are defined here. They're read through out the entire program.
//Local variables are defined within each function. They can only be changed or read within the function. 

//Records whether the ultrasonic sensor has detected something.
//It's controlled within the DistanceSensor function. It will stay on for four seconds when something is within range. 
//Everytime it registers something it refreshes the timer.
boolean Distance_Close;

//Remembers if trashcan is open or closed. One is open and zero is closed.
bool TrashcanState;

//Monitors for local variables.
int ButtonMonitor;
int DistanceMonitor;
int CloseMonitor;
unsigned long TimeOpenMonitor;
unsigned long TimeUsedMonitor;
int DetectedMonitor;
int TemperatureMonitor;
int HumidityMonitor;

//Sets the pin numbers.
const int DCmotorIn1 = 5;
const int DCmotorIn2 = 6;
const int TrigPin = 8;
const int EchoPin = 7;
const int ButtonPin = 9;
const int rs = 13, en = 12, d4 = 11, d5 = 10, d6 = 22, d7 = 24; //Pin numbers for the LCD.

const char* ssid     = "INSERT NAME HERE";         //SSID of the network (name of the wifi network)
const char* password = "INSERT PASSWORD HERE";     // The password of the Wi-Fi network

//Libraries, make sure to install all of them.
//All libraries can be found inside the Library Manager.
//"Adafruit Unified Sensor Library" must also be installed in order for dht to work even though it is not loaded.
#include <LiquidCrystal.h>
#include "DHT.h"

#include <PubSubClient.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>

#define DHTPIN 3     //Defines which pin should be used for the temperature and humidity sensor
#define DHTTYPE DHT11 //Defines which type of DHT is used. If you have a DHT22 replace DHT11 with DHT22.


DHT dht(DHTPIN, DHTTYPE); //Loads the defined pin and type.

LiquidCrystal lcd(rs, en, d4, d5, d6, d7); //Tells the program which pins are installed.

WiFiClient wifiClient;

PubSubClient client(wifiClient);

void setup() {
// Code that needs to be run once:
Serial.begin(9600); //Starts communication with the monitor. Make sure that the monitor have the same BAUD speed.

//Determine if pins are reading or writing.
pinMode(TrigPin,OUTPUT);
pinMode(EchoPin,INPUT);
pinMode(DCmotorIn1, OUTPUT);
pinMode(DCmotorIn2, OUTPUT);
pinMode(ButtonPin,INPUT);

dht.begin();
lcd.begin(16, 2); //Sets the dimension of the LCD

Serial.begin(115200);         // Start the Serial communication to send messages to the computer
  delay(10);
  Serial.println('\n');
  
  WiFi.begin(ssid, password);             // Connect to the network
  Serial.print("Connecting to ");
  Serial.print(ssid); Serial.println(" ...");

  int i = 0;
  while (WiFi.status() != WL_CONNECTED) { // Wait for the Wi-Fi to connect
    delay(1000);
    Serial.print(++i); Serial.print(' ');
  }
Serial.println('\n');
Serial.println("Connection established!");  
Serial.print("IP address:\t");
Serial.println(WiFi.localIP());         // Send the IP address of the ESP8266 to the computer
  }

void loop() {
// put your main code here, to run repeatedly:
DistanceSensor(); //Constantly checks if someone is near the trashcan
OpenTrashcan(); //Checks if the trashcan should opened.
CloseTrashcan(); //Checks if the trashcan should be closed.
LCD();
TempHumid();

//Creates a table for monitoring all the different variables. 
Serial.print("Trashcan State : ");
Serial.print(TrashcanState);
Serial.print("   ");
Serial.print("Distance_Close: " );
Serial.print(Distance_Close);
Serial.print("   ");
Serial.print("Button: " );
Serial.print(ButtonMonitor);
Serial.print("   ");
Serial.print("Temperature: " );
Serial.print(TemperatureMonitor);
Serial.print("   ");
Serial.print("Humidity: " );
Serial.print(HumidityMonitor);
Serial.print("   ");
Serial.print("Distance: " );
Serial.println(DistanceMonitor);
//Serial.print("   ");
//Serial.print("TimeUsed: " );
//Serial.println(TimeUsedMonitor);


}
